function Dsp = Rx_Channel_Estimation(PARAM,Dsp)
%% input isolation & coefficients

Y     = Dsp.RxSymbol;
Theta = Dsp.RxThetaRxTx;
% PhaseIndex = Dsp.RxPhaseRefIndex;

DataLength = size(Y, 1);

%% extract pilots
PilotIndex = PARAM.PilotIndex;
PilotIndex(PilotIndex > DataLength) = [];

Y = Y(PilotIndex, :);

%% extract reference phase
RefIndex   = PARAM.PilotRefIndex;
IndexStep  = PARAM.CellLength * PARAM.Nt;
PhaseIndex = RefIndex : IndexStep : DataLength;

Theta = Theta(PhaseIndex, :);

%expand Theta for Pilot structure.
Theta = reshape(Theta, 1, []);
Theta = repmat(Theta, PARAM.Nt, 1);
Theta = reshape(Theta, [], PARAM.Nr + PARAM.Nt);

%% sort out data
[DataLength, ~] = size(Y);
X = repmat(PARAM.PilotSeed, DataLength / PARAM.Nt, 1);

ThetaRx = Theta(:, 1 : PARAM.Nr);
ThetaTx = Theta(:, PARAM.Nr + 1 : end);

%% H estimation
Y = Y .* exp(- 1i * ThetaRx);
X = X .* exp(  1i * ThetaTx);

HEst = Y.' * (X.')';
HEst = HEst ./ DataLength; %normalization

%% output
Dsp.RxHEst = HEst;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% test plot
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Delta Theta control
CutHead = Dsp.RxCutHead / PARAM.CellLength / PARAM.Nt;
ThetaLength = size(Dsp.RxThetaRxTx, 1);
TestIndex = CutHead + 1 : CutHead + ThetaLength;

%Tx
ThetaTxRef = Dsp.ChThetaTx - Dsp.ChThetaTx(:, RefIndex);
ThetaTxRef = ThetaTxRef(TestIndex, :); %cut unstable
ThetaTxRef = ThetaTxRef(PilotIndex, :); %pilot location
ThetaTxRef = ThetaTxRef(RefIndex : PARAM.Nt : end, :); %ref location

ThetaTxEst = Dsp.RxThetaRxTx(:, PARAM.Nr + 1 : end);
ThetaTxEst = ThetaTxEst(PilotIndex, :); %pilot location
ThetaTxEst = ThetaTxEst(RefIndex : PARAM.Nt : end, :); %ref location

DeltaTx = mean(ThetaTxEst) - mean(ThetaTxRef);

%Rx
ThetaRxRef = Dsp.ChThetaRx + Dsp.ChThetaTx(:, RefIndex);
ThetaRxRef = ThetaRxRef(TestIndex, :); %cut unstable
ThetaRxRef = ThetaRxRef(PilotIndex, :); %pilot location
ThetaRxRef = ThetaRxRef(RefIndex : PARAM.Nt : end, :); %ref location

ThetaRxEst = Dsp.RxThetaRxTx(:, 1 : PARAM.Nr);
ThetaRxEst = ThetaRxEst(PilotIndex, :); %pilot location
ThetaRxEst = ThetaRxEst(RefIndex : PARAM.Nt : end, :); %ref location

DeltaRx = mean(ThetaRxEst) - mean(ThetaRxRef);

DeltaAngleMat = DeltaRx.' + DeltaTx;
DeltaHMat = exp(1i * DeltaAngleMat);

%% H calibration
HEstTest = HEst .* DeltaHMat;

%% plot
if PARAM.PlotOn
  HTest = zeros(PARAM.Nr, PARAM.Nt, DataLength / PARAM.Nt);
  for i1 = 1 : PARAM.Nt : DataLength
    i1Reg = (i1 - 1) / PARAM.Nt + 1;
    HTest(:, :, i1Reg) = Y(i1 : i1 + PARAM.Nt - 1, :).' ...
              * (X(i1 : i1 + PARAM.Nt - 1, :).')';
  end
  HTest = HTest ./ PARAM.Nt;%normalization
  
  HTest = HTest .* DeltaHMat;
  HTest = reshape(HTest, [],1,1);
  
  figure; 
  plot(real(HTest)   , imag(HTest)   , 'g.'); hold on;
  plot(real(HEstTest), imag(HEstTest), 'ro'); hold on;
%   plot(real(Dsp.ChH) , imag(Dsp.ChH) , 'k+'); hold on;
  axis equal;
  grid on; grid minor;
end

title('Channel Estimation (phases are not rotated back yet)');

end %function
