function Dsp = Rx_HAbs_Estimation(PARAM,Dsp)
%% input isolation
PilotRef = PARAM.PilotSeed;
sRx      = Dsp.ChSymbol;
SnrEst   = Dsp.RxSnrEstAvg;

%% pilot extraction
PilotIndex = PARAM.PilotIndex;

sPilot = sRx(PilotIndex, :);
PilotLength = size(sPilot, 1);

%% |H| estimation. 
% HAbsEst > HAbsRef when HAbsRef->0  . This is dominated by AWGN.
% HAbsEst < HAbsRef when HAbsRef->inf. This is dominated by phase noise.
% The above phenomenon will slightly degrade phase estimation accuracy.

HAbs2 = zeros(PARAM.Nr, PARAM.Nt);
for i1 = 1 : PARAM.Nt : PilotLength
  HAbsTemp = sPilot(i1 : PARAM.Nt + i1 - 1, :).' * PilotRef' ./ PARAM.Nt;
  HAbsTemp = abs(HAbsTemp);
  
  HAbs2 = HAbs2 + HAbsTemp.^2;
end

HAbs2 = HAbs2 ./ (PilotLength / PARAM.Nt); %normalize 
HAbs2 = HAbs2 - 1 / SnrEst / PARAM.Nt; %cancel noise
HAbs2(HAbs2 < 0) = 0; %set negative data to 0.

HAbs = sqrt(HAbs2);

%% calculate output
Dsp.RxHAbsEst = HAbs;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% test figure & data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% H test
% HAbsRef = abs(Dsp.ChH);
% HCompare = HAbs ./ HAbsRef;
% HDelta = max(HCompare, [], 'all') - min(HCompare, [], 'all');

end %function
