%use MIMO MMSE algorithm.

function Dsp = Rx_Phase_Recovery_WLLS(PARAM,Dsp)
%% input isolation & coefficients
PilotRef = PARAM.PilotSeed;
sRx      = Dsp.ChSymbol;
HAbs     = Dsp.RxHAbsEst;
SnrEst   = Dsp.RxSnrEstAvg;

RefIndex = PARAM.PilotRefIndex;

%% pilot extraction
PilotIndex = PARAM.PilotIndex;

sPilot = sRx(PilotIndex, :);
PilotLength = size(sPilot, 1);

%% Theta. on-shot estimation. A = angle(Y * X');
A = zeros(PilotLength / PARAM.Nt, PARAM.Nr * PARAM.Nt);
for i1 = 1 : PARAM.Nt : PilotLength
  i1Cnt = (i1 - 1) / PARAM.Nt + 1;
  AReg = angle(sPilot(i1 : PARAM.Nt + i1 - 1, :).' * PilotRef');
  AReg = reshape(AReg, 1, []);
  A(i1Cnt , :) = AReg;
end
% A = unwrap(A, [], 1);
A = Unwrap_Mean(A,20,pi);
% A = A - A(1,:);%force initial zero for test purpose.

%% generate C matrix.
%arrangement of X: {R_1; ...; R_Nr; T_1; ...; T_Nt}
CL = repmat(eye(PARAM.Nr), PARAM.Nt, 1);
CR = reshape(eye(PARAM.Nt), 1, []);
CR = repmat(CR, PARAM.Nr, 1);
CR = reshape(CR, PARAM.Nt * PARAM.Nr, PARAM.Nt);
CR(:,RefIndex) = [];%0; 
  %Force reference ThetaTx as 0 to ensure full rank.
C = [CL, CR]; %A + n = C * ThetaRxTx
CInv = pinv(C); 

%generate noise vector
HAbs = reshape(HAbs,[],1);
nInv = HAbs;

%weighted algorithm.
ANorm = A .* nInv.';
CNorm = C .* nInv;
CInvNorm = pinv(CNorm);

%calculate ThetaRxTx vector
ThetaRxTx = (CInvNorm * ANorm.').';
ThetaRxTx = [ThetaRxTx(:, 1 : PARAM.Nr + RefIndex - 1), ...
             zeros(size(ThetaRxTx, 1), 1), ...
             ThetaRxTx(:, PARAM.Nr + RefIndex : end)];

%calculate SigmaN2
SigmaN2 = sum(CInvNorm.^2, 2)' .* 1 / (2 * SnrEst * PARAM.Nt);
  %PARAM.Nt comes from the inverse calculation at noise vector.
SigmaN2 = [SigmaN2(:, 1 : PARAM.Nr + RefIndex - 1), ...
           zeros(size(SigmaN2, 1), 1), ...
           SigmaN2(:, PARAM.Nr + RefIndex : end)];
         
%% calculate output
Dsp.RxThetaRxTx = ThetaRxTx;
Dsp.RxThetaRxTxOneShot = Dsp.RxThetaRxTx;
Dsp.RxThetaSigmaN2 = SigmaN2;

% Dsp.RxCutHead = (RefIndex - 1) * PARAM.CellLength + Dsp.RxCutHead;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% test figure & data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if PARAM.PlotOn
  %% C test
  figure; heatmap(C); title('C');
  figure; heatmap(CInv); title('CInv');
  figure; heatmap(CInvNorm); title('CInvNorm');
  
  %% A test
  figure;
  plot(1:length(A),A,'.-');hold on;
  title('Elementwise Theta in H without WLLS');
end

%% ThetaTx test
if PARAM.PlotOn; figure; end
ThetaTxFig = Dsp.RxThetaRxTx(:, PARAM.Nr + 1 : end);
Sigma2Tx = zeros(1, PARAM.Nt);
for i1 = 1 : PARAM.Nt
  %reference
  ThetaTxRef = Dsp.ChThetaTx(:, i1) - Dsp.ChThetaTx(:, RefIndex);
  ThetaTxRef = ThetaTxRef(PilotIndex, :); %pilot location
  ThetaTxRef = ThetaTxRef(RefIndex : PARAM.Nt : end, :); 
    %mid pilot frame location
  if PARAM.PlotOn
    plot(1 : length(ThetaTxRef), ThetaTxRef, 'b.-'); hold on;
  end
  
  %estimation
  DeltaTx = mean(ThetaTxFig(:,i1)) - mean(ThetaTxRef);
  ThetaTxEst = ThetaTxFig(:, i1) - DeltaTx;
  
  if PARAM.PlotOn
    plot(1 : length(ThetaTxEst), ThetaTxEst, 'r.-'); hold on;
  end
  
  Sigma2Tx(1, i1) = mean(abs(ThetaTxRef - ThetaTxEst) .^ 2, 1);
end
Dsp.TestSigma2ThetaTxOneShot = Sigma2Tx * PARAM.Nt / (PARAM.Nt - 1);
  %clear reference.
title('ThetaTx after WLLS (bias subtracted)');

%% ThetaRx test
if PARAM.PlotOn; figure; end
ThetaRxFig = Dsp.RxThetaRxTx(:, 1 : PARAM.Nr);
Sigma2Rx = zeros(1, PARAM.Nr);
for i1 = 1 : PARAM.Nr
  %reference
  ThetaRxRef = Dsp.ChThetaRx(:, i1) + Dsp.ChThetaTx(:, RefIndex);
  ThetaRxRef = ThetaRxRef(PilotIndex, :); %pilot location
  ThetaRxRef = ThetaRxRef(RefIndex : PARAM.Nt : end, :); 
    %mid pilot frame location
    
  if PARAM.PlotOn
    plot((1 : length(ThetaRxRef)), ThetaRxRef, 'b.-'); hold on;
  end
  
  %estimation
  DeltaRx = mean(ThetaRxFig(:,i1)) - mean(ThetaRxRef);
  ThetaRxEst = ThetaRxFig(:, i1) - DeltaRx;
  
  if PARAM.PlotOn
    plot(1 : length(ThetaRxEst), ThetaRxEst, 'r.-'); hold on;
  end
  
  Sigma2Rx(1, i1) = mean(abs(ThetaRxRef - ThetaRxEst) .^ 2, 1);
end
Dsp.TestSigma2ThetaRxOneShot = Sigma2Rx;
title('ThetaRx after WLLS (bias subtracted)');

end %function
