function Dsp = Rx_Phase_Recovery_Wiener(PARAM,Dsp)

%% input isolation
SigmaP2 = 2 * pi * (2 * PARAM.LineWidth) ... %2x is compared to reference
        / (PARAM.BaudRate / PARAM.Nt / PARAM.CellLength);
SigmaN2 = Dsp.RxThetaSigmaN2;

[DataLength, DataWidth] = size(Dsp.RxThetaRxTx);

%% Phase noise matrix
TapL = PARAM.WienerTapHalf;
TapR = TapL;
PhaseMat = zeros(TapL + TapR + 1, TapL + TapR + 1);
for i1 = 1 : TapL
  PhaseMat(i1, 1 : i1) = TapL - i1 + 1;
  PhaseMat(1 : i1, i1) = TapL - i1 + 1;
end

for i1 = TapL + 2 : TapL + TapR + 1
  PhaseMat(i1, i1 : end) = i1 - TapL - 1;
  PhaseMat(i1 : end, i1) = i1 - TapL - 1;
end

%% AWGN matrix
NoiseMat = eye(TapL + TapR + 1);

%% wiener filtering
ThetaRxTxReg = zeros(DataLength, DataWidth);
for i1 = 1 : DataWidth
  if SigmaN2(1,i1) == 0 %reference group
    w = ones(1, TapL + TapR + 1);
  else %SigmaN2(1,i1) ~= 0
    %auto-correlation matrix
    Cy = SigmaP2 * PhaseMat + SigmaN2(1,i1) * NoiseMat;

    %wiener coefficient
    OneVector = ones(1, TapL + TapR + 1);
    w = (OneVector / Cy) / (OneVector / Cy * OneVector');
  end

  %filtering
  ThetaRxTxReg(:,i1) = fftfilt(w, Dsp.RxThetaRxTx(:,i1));
end

%output
Dsp.RxThetaRxTx = ThetaRxTxReg(TapL + TapR + 1 : end, :);
Dsp.RxThetaRxTxWiener = Dsp.RxThetaRxTx;
Dsp.RxCutHead = TapL * PARAM.CellLength * PARAM.Nt + Dsp.RxCutHead;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% test figure & data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

RefIndex = PARAM.PilotRefIndex;
PilotIndex = PARAM.PilotIndex;

% PilotIndex = (1:PARAM.Nt)';
% PilotIndex = repmat(PilotIndex, 1, ...
%                     size(Dsp.ChSymbol, 1) / PARAM.CellLength / PARAM.Nt);
% PilotIndex = PilotIndex ...
%            + (1 : PARAM.CellLength * PARAM.Nt : size(Dsp.ChSymbol, 1)) - 1;
% PilotIndex = reshape(PilotIndex, [], 1);

%% ThetaTx test
if PARAM.PlotOn; figure; end
ThetaTxFig = Dsp.RxThetaRxTx(:, PARAM.Nr + 1 : end);
Sigma2Tx = zeros(1, PARAM.Nt);
for i1 = 1 : PARAM.Nt
  %reference
  ThetaTxRef = Dsp.ChThetaTx(:, i1) - Dsp.ChThetaTx(:, RefIndex);
  ThetaTxRef = ThetaTxRef(PilotIndex, :); %pilot location
  ThetaTxRef = ThetaTxRef(RefIndex : PARAM.Nt : end, :); 
    %mid pilot frame location
  ThetaTxRef = ThetaTxRef(TapL + 1 : end - TapR, :); %truncate adjustment
    
  if PARAM.PlotOn
    plot((1 : length(ThetaTxRef)), ThetaTxRef, 'b.-'); hold on;
  end
  
  %estimation
  DeltaTx = mean(ThetaTxFig(:,i1)) - mean(ThetaTxRef);
  ThetaTxEst = ThetaTxFig(:, i1) - DeltaTx;
  
  if PARAM.PlotOn
    plot(1 : length(ThetaTxFig), ThetaTxEst, 'r.-'); hold on;
  end
  
  Sigma2Tx(1, i1) = mean(abs(ThetaTxRef - ThetaTxEst) .^ 2, 1);
end
Dsp.TestSigma2ThetaTxWiener = Sigma2Tx * PARAM.Nt / (PARAM.Nt - 1);
  %clear reference
title('ThetaTx after Wiener filter (bias subtracted)');

%% ThetaRx test
if PARAM.PlotOn; figure; end
ThetaRxFig = Dsp.RxThetaRxTx(:, 1 : PARAM.Nr);
Sigma2Rx = zeros(1, PARAM.Nr);
for i1 = 1 : PARAM.Nr
  %reference
  ThetaRxRef = Dsp.ChThetaRx(:, i1) + Dsp.ChThetaTx(:, RefIndex);
  ThetaRxRef = ThetaRxRef(PilotIndex, :); %pilot location
  ThetaRxRef = ThetaRxRef(RefIndex : PARAM.Nt : end, :); 
    %mid pilot frame location
  ThetaRxRef = ThetaRxRef(TapL + 1 : end - TapR, :); %truncate adjustment

  if PARAM.PlotOn
    plot((1 : length(ThetaRxRef)), ThetaRxRef, 'b.-'); hold on;
  end
  
  %estimation
  DeltaRx = mean(ThetaRxFig(:,i1)) - mean(ThetaRxRef);
  ThetaRxEst = ThetaRxFig(:, i1) - DeltaRx;
  
  if PARAM.PlotOn
    plot(1 : length(ThetaRxFig), ThetaRxEst, 'r.-'); hold on;
  end
  
  Sigma2Rx(1, i1) = mean(abs(ThetaRxRef - ThetaRxEst) .^ 2, 1);
end

Dsp.TestSigma2ThetaRxWiener = Sigma2Rx;

title('ThetaRx after Wiener filter (bias subtracted)');

end %function
