%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project     : PON_HD
% Module      : SymbolEstimation
% Description : Symbol Estimation package.
% References  : 
%
%Version|Date    |author|Description
%-------|--------|------|-------------------------------------------------------
%01.01  |20210403|L,YM  |Initial version
%-------|--------|------|-------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Dsp = SymbolEstimation(PARAM,Dsp)

%% input isolation
Theta   = Dsp.RxThetaRxTx;
ThetaRx = Theta(:, 1 : PARAM.Nr);
ThetaTx = Theta(:, PARAM.Nr + 1 : end);

%% main


switch PARAM.MimoAlgorithm
  case 'MLD'
    %cut mode
    CutLen = min(1e2,size(Dsp.RxSymbol, 1));
    Dsp.RxSymbol = Dsp.RxSymbol(1:CutLen, :);
    for i1 = 1 : CutLen
%     for i1 = 1 : size(Dsp.RxSymbol, 1) %std mode
      H = diag(exp(1i * ThetaRx(i1,:))) * Dsp.RxHEst ...
        * diag(exp(1i * ThetaTx(i1,:)));
      Dsp.RxSymbol(i1,:) = SymbolEstimation_MLD(PARAM,Dsp.RxSymbol(i1,:),H);
    end
  case 'MMSE'
    %Rx rotation
    Dsp.RxSymbol = Dsp.RxSymbol .* exp(- 1i * ThetaRx); 
    
    %estimation
    Dsp = SymbolEstimation_MMSE(PARAM,Dsp);
    
    %Tx rotation
    Dsp.RxSymbol = Dsp.RxSymbol .* exp(- 1i * ThetaTx);
  otherwise
      fprintf(2,'\n Error: MIMO Algorithm is not supported!')
end


%% test plot
if PARAM.PlotOn
  figure;
  DataPlot = Dsp.RxSymbol;
  %plot function
  for i1 = 1 : PARAM.Nt
    ax = subplot(1, PARAM.Nt, i1);
    plot(ax, real(DataPlot(:,i1)), imag(DataPlot(:,i1)), 'g.'); hold on;
    
    grid on; grid minor;
    axis equal
    title(['MMSE Tx',num2str(i1)])
  end
end

end%function