%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project     : PON_HD
% Module      : SymbolEstimation_MLD
% Description : Symbol Estimation. Use joint MLD detector.
% References  : 
%   [R1] Proakis. Digital Communications 5th ed.
%
%Version|Date    |author|Description
%-------|--------|------|-------------------------------------------------------
%01.01  |20201007|L,YM  |Initial version
%-------|--------|------|-------------------------------------------------------
%01.02  |20210127|L,YM  |Iterative version. Suitable for higher order
%       |        |      |constellation(m>16, N.Tx>2)
%-------|--------|------|-------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DataOut = SymbolEstimation_MLD(PARAM,DataIn,H)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% input isolation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% H      = Dsp.RxHEst;
% DataIn = Dsp.RxSymbol;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% coeffients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DataLength = size(DataIn,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate reference Hs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s = PARAM.FormatMap; %initialization. And include the case N_Tx = 1.

for i1 = 2 : 1 : PARAM.Nt
  s_reg = s;%register data
  s = zeros( PARAM.SymbolFormat ^ i1 , i1 );%renew data
  for i2  = 1 : 1 : PARAM.SymbolFormat
    s( (i2-1) * length(s_reg) + 1 : i2 * length(s_reg) , : ) = ...
      [s_reg , repmat(PARAM.FormatMap(i2),length(s_reg),1)];
  end
end

Hs = H * s.';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SymbolEstimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DeltaMin = Inf(DataLength, 1);
% IndexMin = zeros(DataLength, 1); %initialize IndexMin for debugging
DataOut  = zeros(DataLength, PARAM.Nt);
for i1 = 1 : 1 : PARAM.SymbolFormat ^ PARAM.Nt
  HsMatrix = repmat(Hs(:,i1).' , DataLength , 1);
  DeltaMatrix = abs(DataIn - HsMatrix) .^ 2;
  Delta = sum(DeltaMatrix,2);
  % IndexMin(Delta < DeltaMin) = i1; %renew IndexMin for debugging
  DataOut(Delta < DeltaMin,:)  = ...
    repmat( s(i1,:) , sum(Delta < DeltaMin) , 1); %renew DataOut
  
  DeltaMin(Delta < DeltaMin) = Delta(Delta < DeltaMin); %renew DeltaMin
end

% Dsp.SymbolHard = DataOut;

end%function